////////////////////Utils
function quote(str) {
    str = String(str); // to make sure it is a string
    var c, i, l = str.length, o = "'";
    for (i = 0; i < l; i++) {
        c = str.charAt(i);
        if (c >= ' ') {
            if (c === '\\' || c === "'") {
                o += '\\';
            }
            o += c;
        } else {
            switch (c) {
                case '\b':
                    o += '\\b';
                    break;
                case '\f':
                    o += '\\f';
                    break;
                case '\n':
                    o += '\\n';
                    break;
                case '\r':
                    o += '\\r';
                    break;
                case '\t':
                    o += '\\t';
                    break;
                default:
                    c = c.charCodeAt();
                    o += '\\u00' + Math.floor(c / 16).toString(16) + (c % 16).toString(16);
            }
        }
    }
    return o + "'";
};

function esc(obj) {
	if (obj == null) return "null";
    if (obj.replace) { // String 
    	return quote(obj);
    } else if (obj.getTime) { // Date Object
        obj = obj.getTime();
        return obj;                            
    } else { // number
    	return obj;
    }
}

function list(ids) {
	if (ids == null || ids.length == 0) return "()";
	var buf = ids.split('|');
	return "('" + buf.join("','") + "')";
}

////////////////// Date Utils
var DAY = 24*60*60*1000; // in milliseconds

// returns the current time in EPOCH time, in milliseconds
function now() {
	return new Date().getTime();
}

// returns today start, added by nbDays. at 00:00AM, in milliseconds
function today(nbDays) { 
	if (nbDays == null) nbDays = 0;
    var now = new Date(); //current date and time zone
    var todayStart = new Date(now.getFullYear(), now.getMonth(), now.getDate(), 0, 0, 0, 0)
    return todayStart.getTime() + nbDays*DAY;
}
 
///////////////////////////// User class : new to API Level 6
//helper function returning the current username
function user() { 
	return User.getName(); 
}

function User() {}

User.getName = function() {
	// WARNING : it is ESSENTIAL to have the "" + before : it will force the conversion of the Object type return by Settigns.getUsername into a javascript string
	// and this is required with "{user}" extrapolation calling esc() then, quote();
	return "" + Settings.getUsername();
}

User.isEnterprise = function() {
	var license = Settings.get("license");
	return (license == "3");
}

//////////////////////////
function Config () {}
Config.tables = new Array();

function R() {}

// Tablet friendly apps should redefine this function in their Code
// we define it here just in case the app does not redefine it, to avoid runtime script error
function leftPane() { List.show("leftpane"); }

//////////////////////////

function Query() {}

Query.insert = function(table, values) { return _Query.insert(table, values); }
Query.update = function(table, values, where) { _Query.update(table, values, where); }
Query.updateId = function(table, recordId, columnName, value) { _Query.updateId(table, recordId, columnName, String(value)); }
Query.deleteId = function(table, recordId) { _Query.deleteId(table, recordId); }
Query.deleteWhere = function(table, where) { _Query.deleteWhere(table, where); }

Query.count = function(table, where) { return _Query.count(table, where); }
Query.options = function(table, where) { return _Query.options(table, where); }
Query.ids = function(table, where) { return _Query.ids(table, where); }

Query.select = function(table, columns, where, orderby) { return JSON.parse(_Query.select(table, columns, where, orderby)); }
Query.selectFormat = function(table, columns, where, orderby) { return JSON.parse(_Query.selectFormat(table, columns, where, orderby)); }
Query.selectId = function(table, id) { return JSON.parse(_Query.selectId(table, id)); }
Query.selectNearest = function(table, geo, range) { return JSON.parse(_Query.selectNearest(table, geo, range)); }

Query.rss = function(url, minutes) {if (minutes == null) minutes=60; return JSON.parse(_Query.rss(url, minutes)); }
Query.search = function(table, search, columns, where, orderby) { return JSON.parse(_Query.search(table, search, columns, where, orderby)); }

Query.names = function(table, ids) {
	if (ids == '') return '';
	var buf = new Array();
	var items = Query.select(table, "name", "id IN " + list(ids));
	for (var i = 0; i < items.length; i++) {
		buf.push(items[i].name);
	}
	return buf.join(", ");
}

////////////////////////////

function List() { }

List._items = new Array();

// private method
List._addItem = function(type, label, value, onclick, style) {
	var item = new Object();
    item.type = type;
    if (label != null) item.label = String(label);
    if (value != null) item.value = String(value);
    if (onclick != null) item.onclick = String(onclick);
    if (style != null) item.style = String(style);
    List._items.push(item);
}

List._addEditItem = function(editStyle, id, label, value, onchange, options, onnew) {
	var item = new Object();
	item.editStyle = editStyle;
	if (id != null) item.id = String(id);
	if (label != null) item.label = String(label);
	if (value != null)  item.value = String(value);
	if (onchange != null)  item.onclick = String(onchange);
	if (options != null)  item.options = String(options);
	if (onnew != null)  item.onnew = String(onnew);
	List._items.push(item);
}

List._removeLastItemIfHeader = function() {
	if (List._items.length > 0) {
		if (List._items[List._items.length-1].type == 3) {
			List._items.pop();
		}
	}
}

List._flush = function() {
	if (List._items.length > 0) {
		_List.addItems(JSON.stringify(List._items));
		List._items.length = 0;
	}
}

///////// Public Methods

List.add = function(array, onclick, style) {
	if (array.push != null) { // string
		var text = array.shift();
		if (array.length > 1) {
			if (style == null) style = "";
			style += ";label:" + array.pop();
		}
		List._addItem(2, text, array.join("\n"), onclick, style);
	} else { // string
		List._addItem(0, array, null, onclick, style);
	}
}

List.addLabel = function(label, value, onclick, style) {
    if (value == null || value.length == 0) return;
	List._addItem(1, label, value, onclick, style);
}

List.addHeader = function(array, width) {
	if (array == null) return;
	var title = (array.push != null) ? array[0] : array;
	List._removeLastItemIfHeader();
	List._addItem(3, title, null, null, null);
}

List.addButton = function(text, onclick, style) {
	List._addItem(4, text, null, onclick, style);
}

List.addImage = function(imgurl, onclick, style) {
	List._addItem(5, null, imgurl, onclick, style);
}

List.addItemRating = function(label, value, onclick, style) {
    List._addItem(6, label, value, onclick, style);
}

List.addItemColor = function(label, value, onclick, style) {
    List._addItem(7, label, value, onclick, style);
}
  
List.addTitle = function(text, subtitle, onclick, style) {
	List._addItem(9, text, subtitle, onclick, style);
}

List.addBubble = function(text, subtitle, onclick, style) {
	List._addItem(11, text, subtitle, onclick, style);
}

List.addTextBox = function(id, label, value, onchange, style) {
	if (style == null) style = "text";
	List._addEditItem(style, id, label, value, onchange);
}

List.addCheckBox = function(id, label, checked, onchange) {
	var value = checked ? "1" : "0";
	List._addEditItem("checkbox", id, label, value, onchange);
}

List.addToggleBox = function(id, label, value, onchange, options) {
	List._addEditItem("toggle", id, label, value, onchange, options);
}

List.addComboBox = function (id, label, value, onchange, options, onnew) {
	List._addEditItem("combobox", id, label, value, onchange, options, onnew);
}

List.addComboBoxMulti = function(id, label, value, onchange, options, onnew) {
	List._addEditItem("comboboxmulti", id, label, value, onchange, options, onnew);
}

List.addSignatureBox = function(id, label, value, onchange) {
	if (value != "" && label != "") List.addHeader(label);
	List._addEditItem("signature", id, label, value, onchange);
}

List.addBarcodeBox = function(id, label, value, onchange) {
	List._addEditItem("barcode", id, label, value, onchange);
}

List.addGeoBox = function(id, label, value, onchange, address) {
	List._addEditItem("geo", id, label, value, onchange, address);
}

List.bindItems = function(table, column, where, onclick, oncontext) {
	List._flush();
	_List.bindItems(table, column, where, onclick, oncontext);
}

List.bindItemSubtitles = function(table, columns, where, orderby, onclick, oncontext) {
	List._flush();
	_List.bindItemSubtitles(table, columns, where, orderby, onclick, oncontext);
}

List.bindNearbyItems = function(table, geo, range, onclick, style) {
	List._flush();
	_List.bindNearbyItems(table, geo, range, onclick, style);
}

List.getValue = function(itemId) {
   return _List.getValue(itemId);
}

List.setValue = function(id, value, options) {
	_List.setValue(id, value, options);
}

List.setEmpty = function(text) {
	_List.setEmpty(text);
}

List.setStyle = function(value) {
	_List.setStyle(value);
}

List.show = function(style) {
	List._removeLastItemIfHeader();
	List._flush();
   _List.show(style);
}

/////////////////////////////////  TODO : Deprecate at some point
 
List.addItemTitle = function(text, subtitle, onclick, style) {
	List.addTitle(text, subtitle, onclick, style);
}

List.addItemLabel = function(label, value, onclick, style) {
	List.addLabel(label, value, onclick, style);
}

List.addItem = function(text, onclick, style) {
	List._addItem(0, text, null, onclick, style);
}

List.addItemSubtitle = function(text, subtitle, onclick, style) {
	List._addItem(2, text, subtitle, onclick, style);
}

///////////////////////

function History() {}
History.back = function() {_History.back();}
History.next = function() {_History.next();}
History.previous = function() {_History.previous();}
History.reload = function(funcCall) {_History.reload(funcCall);}
History.get = function() {return _History.get();}

History.add = function(funcCall) {_History.add(funcCall);}
History.remove = function(count) {_History.remove(count);}

History.redirect = function(funcCall) {_History.replace(funcCall);eval(funcCall);}

// TODO : deprecate at some point
History.replace = function(funcCall, previous) {
	if (previous != null) History.add(previous);
	History.redirect(funcCall);
}

//////////////////////

function Calendar() {}
Calendar.setCallback = function(callback) {
	_List.setCallback(callback);
}

Calendar.addItem = function(title, date) {
	List.addItemLabel(title, String(date));
}

Calendar.show = function() {
	List.show("calendar");
}




